@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -99,7 +99,9 @@ namespace Orthanc
       void StartMultipart(const std::string& subType,
                           const std::string& contentType);
 
-      void SendMultipartItem(const void* item, size_t length);
+      void SendMultipartItem(const void* item, 
+                             size_t length,
+                             const std::map<std::string, std::string>& headers);
 
       void CloseMultipart();
 
@@ -114,7 +116,6 @@ namespace Orthanc
     StateMachine stateMachine_;
     bool         isDeflateAllowed_;
     bool         isGzipAllowed_;
-    bool         describeErrors_;
 
     HttpCompression GetPreferredCompression(size_t bodySize) const;
 
@@ -123,8 +124,7 @@ namespace Orthanc
                bool isKeepAlive) : 
       stateMachine_(stream, isKeepAlive),
       isDeflateAllowed_(false),
-      isGzipAllowed_(false),
-      describeErrors_(true)
+      isGzipAllowed_(false)
     {
     }
 
@@ -148,16 +148,6 @@ namespace Orthanc
       return isGzipAllowed_;
     }
 
-    void SetDescribeErrorsEnabled(bool enabled)
-    {
-      describeErrors_ = enabled;
-    }
-
-    bool IsDescribeErrorsEnabled() const
-    {
-      return describeErrors_;      
-    }
-
     void SendStatus(HttpStatus status,
 		    const char* message,
 		    size_t messageSize);
@@ -214,11 +204,11 @@ namespace Orthanc
       stateMachine_.StartMultipart(subType, contentType);
     }
 
-    void SendMultipartItem(const std::string& item);
-
-    void SendMultipartItem(const void* item, size_t size)
+    void SendMultipartItem(const void* item, 
+                           size_t size,
+                           const std::map<std::string, std::string>& headers)
     {
-      stateMachine_.SendMultipartItem(item, size);
+      stateMachine_.SendMultipartItem(item, size, headers);
     }
 
     void CloseMultipart()
