@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -36,7 +36,6 @@
 #include "MongooseServer.h"
 
 #include "../Logging.h"
-#include "../OrthancException.h"
 #include "../ChunkedBuffer.h"
 #include "HttpToolbox.h"
 #include "mongoose.h"
@@ -583,7 +582,6 @@ namespace Orthanc
 
     MongooseOutputStream stream(connection);
     HttpOutput output(stream, that->IsKeepAliveEnabled());
-    output.SetDescribeErrorsEnabled(that->IsDescribeErrorsEnabled());
 
     // Check remote calls
     if (!that->IsRemoteAccessAllowed() &&
@@ -599,8 +597,11 @@ namespace Orthanc
     for (int i = 0; i < request->num_headers; i++)
     {
       std::string name = request->http_headers[i].name;
+      std::string value = request->http_headers[i].value;
+
       std::transform(name.begin(), name.end(), name.begin(), ::tolower);
-      headers.insert(std::make_pair(name, request->http_headers[i].value));
+      headers.insert(std::make_pair(name, value));
+      VLOG(1) << "HTTP header: [" << name << "]: [" << value << "]";
     }
 
     if (that->IsHttpCompressionEnabled())
@@ -647,9 +648,10 @@ namespace Orthanc
     const IIncomingHttpRequestFilter *filter = that->GetIncomingHttpRequestFilter();
     if (filter != NULL)
     {
-      if (!filter->IsAllowed(method, request->uri, remoteIp, username.c_str()))
+      if (!filter->IsAllowed(method, request->uri, remoteIp, username.c_str(), headers))
       {
-        output.SendUnauthorized(ORTHANC_REALM);
+        //output.SendUnauthorized(ORTHANC_REALM);
+        output.SendStatus(HttpStatus_403_Forbidden);
         return;
       }
     }
@@ -729,18 +731,18 @@ namespace Orthanc
       {
         if (that->HasHandler())
         {
-          found = that->GetHandler().Handle(output, RequestOrigin_Http, remoteIp, username.c_str(), 
+          found = that->GetHandler().Handle(output, RequestOrigin_RestApi, remoteIp, username.c_str(), 
                                             method, uri, headers, argumentsGET, body.c_str(), body.size());
         }
       }
       catch (boost::bad_lexical_cast&)
       {
-        throw OrthancException(ErrorCode_BadParameterType, HttpStatus_400_BadRequest);
+        throw OrthancException(ErrorCode_BadParameterType);
       }
       catch (std::runtime_error&)
       {
         // Presumably an error while parsing the JSON body
-        throw OrthancException(ErrorCode_BadRequest, HttpStatus_400_BadRequest);
+        throw OrthancException(ErrorCode_BadRequest);
       }
 
       if (!found)
@@ -751,22 +753,17 @@ namespace Orthanc
     catch (OrthancException& e)
     {
       // Using this candidate handler results in an exception
-      LOG(ERROR) << "Exception in the HTTP handler: " << e.What();
-
-      Json::Value message = Json::objectValue;
-      message["HttpError"] = EnumerationToString(e.GetHttpStatus());
-      message["HttpStatus"] = e.GetHttpStatus();
-      message["Message"] = e.What();
-      message["Method"] = EnumerationToString(method);
-      message["OrthancError"] = EnumerationToString(e.GetErrorCode());
-      message["OrthancStatus"] = e.GetErrorCode();
-      message["Uri"] = request->uri;
-
-      std::string info = message.toStyledString();
-
       try
       {
-	output.SendStatus(e.GetHttpStatus(), info);
+        if (that->GetExceptionFormatter() == NULL)
+        {
+          LOG(ERROR) << "Exception in the HTTP handler: " << e.What();
+          output.SendStatus(e.GetHttpStatus());
+        }
+        else
+        {
+          that->GetExceptionFormatter()->Format(output, e, method, request->uri);
+        }
       }
       catch (OrthancException&)
       {
@@ -832,7 +829,7 @@ namespace Orthanc
     filter_ = NULL;
     keepAlive_ = false;
     httpCompression_ = true;
-    describeErrors_ = true;
+    exceptionFormatter_ = NULL;
 
 #if ORTHANC_SSL_ENABLED == 1
     // Check for the Heartbleed exploit
@@ -938,7 +935,7 @@ namespace Orthanc
 #if ORTHANC_SSL_ENABLED == 0
     if (enabled)
     {
-      throw OrthancException("Orthanc has been built without SSL support");
+      throw OrthancException(ErrorCode_SslDisabled);
     }
     else
     {
@@ -983,18 +980,20 @@ namespace Orthanc
     LOG(WARNING) << "HTTP compression is " << (enabled ? "enabled" : "disabled");
   }
   
-  void MongooseServer::SetDescribeErrorsEnabled(bool enabled)
-  {
-    describeErrors_ = enabled;
-    LOG(INFO) << "Description of the errors in the HTTP answers is " << (enabled ? "enabled" : "disabled");
-  }
-
   void MongooseServer::SetIncomingHttpRequestFilter(IIncomingHttpRequestFilter& filter)
   {
     Stop();
     filter_ = &filter;
   }
 
+
+  void MongooseServer::SetHttpExceptionFormatter(IHttpExceptionFormatter& formatter)
+  {
+    Stop();
+    exceptionFormatter_ = &formatter;
+  }
+
+
   bool MongooseServer::IsValidBasicHttpAuthentication(const std::string& basic) const
   {
     return registeredUsers_.find(basic) != registeredUsers_.end();
