@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -30,9 +30,11 @@
  **/
 
 
+#include "PrecompiledHeadersServer.h"
 #include "DicomInstanceToStore.h"
 
 #include "FromDcmtkBridge.h"
+#include "OrthancInitialization.h"
 #include "../Core/Logging.h"
 
 #include <dcmtk/dcmdata/dcfilefo.h>
@@ -40,12 +42,6 @@
 
 namespace Orthanc
 {
-  static DcmDataset& GetDataset(ParsedDicomFile& file)
-  {
-    return *reinterpret_cast<DcmFileFormat*>(file.GetDcmtkObject())->getDataset();
-  }
-
-
   void DicomInstanceToStore::AddMetadata(ResourceType level,
                                          MetadataType metadata,
                                          const std::string& value)
@@ -68,17 +64,23 @@ namespace Orthanc
     {
       if (!parsed_.HasContent())
       {
-        throw OrthancException(ErrorCode_NotImplemented);
-      }
-      else
-      {
-        // Serialize the parsed DICOM file
-        buffer_.Allocate();
-        if (!FromDcmtkBridge::SaveToMemoryBuffer(buffer_.GetContent(), GetDataset(parsed_.GetContent())))
+        if (!summary_.HasContent())
         {
-          LOG(ERROR) << "Unable to serialize a DICOM file to a memory buffer";
-          throw OrthancException(ErrorCode_InternalError);
+          throw OrthancException(ErrorCode_NotImplemented);
         }
+        else
+        {
+          parsed_.TakeOwnership(new ParsedDicomFile(summary_.GetConstContent()));
+        }                                
+      }
+
+      // Serialize the parsed DICOM file
+      buffer_.Allocate();
+      if (!FromDcmtkBridge::SaveToMemoryBuffer(buffer_.GetContent(), 
+                                               *parsed_.GetContent().GetDcmtkObject().getDataset()))
+      {
+        LOG(ERROR) << "Unable to serialize a DICOM file to a memory buffer";
+        throw OrthancException(ErrorCode_InternalError);
       }
     }
 
@@ -102,13 +104,21 @@ namespace Orthanc
     if (!summary_.HasContent())
     {
       summary_.Allocate();
-      FromDcmtkBridge::Convert(summary_.GetContent(), GetDataset(parsed_.GetContent()));
+      FromDcmtkBridge::Convert(summary_.GetContent(), 
+                               *parsed_.GetContent().GetDcmtkObject().getDataset(),
+                               ORTHANC_MAXIMUM_TAG_LENGTH,                               
+                               Configuration::GetDefaultEncoding());
     }
     
     if (!json_.HasContent())
     {
       json_.Allocate();
-      FromDcmtkBridge::ToJson(json_.GetContent(), GetDataset(parsed_.GetContent()));
+      FromDcmtkBridge::ToJson(json_.GetContent(), 
+                              *parsed_.GetContent().GetDcmtkObject().getDataset(),
+                              DicomToJsonFormat_Full, 
+                              DicomToJsonFlags_Default,
+                              ORTHANC_MAXIMUM_TAG_LENGTH,
+                              Configuration::GetDefaultEncoding());
     }
   }
 
@@ -196,7 +206,7 @@ namespace Orthanc
         break;
       }
 
-      case RequestOrigin_Http:
+      case RequestOrigin_RestApi:
       {
         result["RemoteIp"] = remoteIp_;
         result["Username"] = httpUsername_;
@@ -230,7 +240,7 @@ namespace Orthanc
   {
     origin_ = call.GetRequestOrigin();
 
-    if (origin_ == RequestOrigin_Http)
+    if (origin_ == RequestOrigin_RestApi)
     {
       remoteIp_ = call.GetRemoteIp();
       httpUsername_ = call.GetUsername();
@@ -240,7 +250,7 @@ namespace Orthanc
   void DicomInstanceToStore::SetHttpOrigin(const char* remoteIp,
                                            const char* username)
   {
-    origin_ = RequestOrigin_Http;
+    origin_ = RequestOrigin_RestApi;
     remoteIp_ = remoteIp;
     httpUsername_ = username;
   }
