@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -171,6 +171,11 @@ namespace Orthanc
 
     void SetPluginsOrigin();
 
+    RequestOrigin GetRequestOrigin() const
+    {
+      return origin_;
+    }
+
     const char* GetRemoteAet() const; 
 
     void SetBuffer(const std::string& dicom)
