@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -40,7 +40,7 @@
 
 namespace Orthanc
 {
-  class IMoveRequestIterator
+  class IMoveRequestIterator : public boost::noncopyable
   {
   public:
     enum Status
@@ -70,7 +70,9 @@ namespace Orthanc
     virtual IMoveRequestIterator* Handle(const std::string& target,
                                          const DicomMap& input,
                                          const std::string& remoteIp,
-                                         const std::string& remoteAet) = 0;
+                                         const std::string& remoteAet,
+                                         const std::string& calledAet,
+                                         uint16_t messageId) = 0;
   };
 
 }
