@@ -1,7 +1,7 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2014 Medical Physics Department, CHU of Liege,
- * Belgium
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
+ * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,10 +36,15 @@
 #include <set>
 #include <json/json.h>
 #include <stdint.h>
+
+#include "../Core/FileStorage/IStorageArea.h"
 #include "../Core/HttpServer/MongooseServer.h"
+#include "../Core/Images/FontRegistry.h"
+#include "../Core/WebServiceParameters.h"
+
 #include "DicomProtocol/RemoteModalityParameters.h"
+#include "IDatabaseWrapper.h"
 #include "ServerEnumerations.h"
-#include "OrthancPeerParameters.h"
 
 namespace Orthanc
 {
@@ -49,6 +54,9 @@ namespace Orthanc
 
   class Configuration
   {
+  private:
+    Configuration();  // Forbidden, this is a static class
+
   public:
     static std::string GetGlobalStringParameter(const std::string& parameter,
                                                 const std::string& defaultValue);
@@ -65,7 +73,7 @@ namespace Orthanc
     static bool LookupDicomModalityUsingAETitle(RemoteModalityParameters& modality,
                                                 const std::string& aet);
 
-    static void GetOrthancPeer(OrthancPeerParameters& peer,
+    static void GetOrthancPeer(WebServiceParameters& peer,
                                const std::string& name);
 
     static void GetListOfDicomModalities(std::set<std::string>& target);
@@ -97,10 +105,24 @@ namespace Orthanc
     static void RemoveModality(const std::string& symbolicName);
 
     static void UpdatePeer(const std::string& symbolicName,
-                           const OrthancPeerParameters& peer);
+                           const WebServiceParameters& peer);
 
     static void RemovePeer(const std::string& symbolicName);
 
     static const std::string& GetConfigurationAbsolutePath();
+
+    static IDatabaseWrapper* CreateDatabaseWrapper();
+
+    static IStorageArea* CreateStorageArea();
+
+    static void GetConfiguration(Json::Value& result);
+
+    static void FormatConfiguration(std::string& result);
+
+    static const FontRegistry& GetFontRegistry();
+
+    static Encoding GetDefaultEncoding();
+
+    static bool HasConfigurationChanged();
   };
 }
