@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -42,7 +42,7 @@ namespace Orthanc
     ServerContext& context_;
 
     bool LookupIdentifier(std::string& publicId,
-                          DicomTag tag,
+                          ResourceType level,
                           const DicomMap& input);
 
   public:
@@ -54,6 +54,8 @@ namespace Orthanc
     virtual IMoveRequestIterator* Handle(const std::string& targetAet,
                                          const DicomMap& input,
                                          const std::string& remoteIp,
-                                         const std::string& remoteAet);
+                                         const std::string& remoteAet,
+                                         const std::string& calledAet,
+                                         uint16_t messageId);
   };
 }
