@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -33,10 +33,13 @@
 #pragma once
 
 #include "../Core/DicomFormat/DicomInstanceHasher.h"
+#include "../Core/Images/ImageAccessor.h"
+#include "../Core/IDynamicObject.h"
 #include "../Core/RestApi/RestApiOutput.h"
 #include "ServerEnumerations.h"
-#include "../Core/Images/ImageAccessor.h"
-#include "../Core/Images/ImageBuffer.h"
+
+class DcmDataset;
+class DcmFileFormat;
 
 namespace Orthanc
 {
@@ -48,22 +51,33 @@ namespace Orthanc
 
     ParsedDicomFile(ParsedDicomFile& other);
 
-    void Setup(const char* content,
-               size_t size);
-
     void RemovePrivateTagsInternal(const std::set<DicomTag>* toKeep);
 
-  public:
-    ParsedDicomFile();  // Create a minimal DICOM instance
+    void UpdateStorageUid(const DicomTag& tag,
+                          const std::string& value,
+                          bool decodeDataUriScheme);
 
-    ParsedDicomFile(const char* content,
+    void InvalidateCache();
+
+    bool EmbedContentInternal(const std::string& dataUriScheme);
+
+  public:
+    ParsedDicomFile(bool createIdentifiers);  // Create a minimal DICOM instance
+
+    ParsedDicomFile(const DicomMap& map);
+
+    ParsedDicomFile(const void* content,
                     size_t size);
 
     ParsedDicomFile(const std::string& content);
 
+    ParsedDicomFile(DcmDataset& dicom);
+
+    ParsedDicomFile(DcmFileFormat& dicom);
+
     ~ParsedDicomFile();
 
-    void* GetDcmtkObject();
+    DcmFileFormat& GetDcmtkObject() const;
 
     ParsedDicomFile* Clone();
 
@@ -74,12 +88,25 @@ namespace Orthanc
 
     void Remove(const DicomTag& tag);
 
-    void Insert(const DicomTag& tag,
-                const std::string& value);
+    void Replace(const DicomTag& tag,
+                 const std::string& utf8Value,
+                 bool decodeDataUriScheme,
+                 DicomReplaceMode mode);
 
     void Replace(const DicomTag& tag,
-                 const std::string& value,
-                 DicomReplaceMode mode = DicomReplaceMode_InsertIfAbsent);
+                 const Json::Value& value,  // Assumed to be encoded with UTF-8
+                 bool decodeDataUriScheme,
+                 DicomReplaceMode mode);
+
+    void Insert(const DicomTag& tag,
+                const Json::Value& value,   // Assumed to be encoded with UTF-8
+                bool decodeDataUriScheme);
+
+    void ReplacePlainString(const DicomTag& tag,
+                            const std::string& utf8Value)
+    {
+      Replace(tag, utf8Value, false, DicomReplaceMode_InsertIfAbsent);
+    }
 
     void RemovePrivateTags()
     {
@@ -107,29 +134,34 @@ namespace Orthanc
     void EmbedImage(const std::string& mime,
                     const std::string& content);
 
-    void ExtractImage(ImageBuffer& result,
-                      unsigned int frame);
-
-    void ExtractImage(ImageBuffer& result,
-                      unsigned int frame,
-                      ImageExtractionMode mode);
-
-    void ExtractPngImage(std::string& result,
-                         unsigned int frame,
-                         ImageExtractionMode mode);
-
     Encoding GetEncoding() const;
 
     void SetEncoding(Encoding encoding);
 
     void ToJson(Json::Value& target, 
-                bool simplify);
+                DicomToJsonFormat format,
+                DicomToJsonFlags flags,
+                unsigned int maxStringLength);
+
+    void HeaderToJson(Json::Value& target, 
+                      DicomToJsonFormat format);
 
     bool HasTag(const DicomTag& tag) const;
 
     void EmbedPdf(const std::string& pdf);
 
     bool ExtractPdf(std::string& pdf);
+
+    void Convert(DicomMap& tags);
+
+    void GetRawFrame(std::string& target, // OUT
+                     std::string& mime,   // OUT
+                     unsigned int frameId);  // IN
+
+    unsigned int GetFramesCount() const;
+
+    static ParsedDicomFile* CreateFromJson(const Json::Value& value,
+                                           DicomFromJsonFlags flags);
   };
 
 }
