@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -32,9 +32,11 @@
 
 #pragma once
 
+#if ORTHANC_PLUGINS_ENABLED == 1
+
 #include "../Include/orthanc/OrthancCPlugin.h"
 
-#include <boost/noncopyable.hpp>
+#include "SharedLibrary.h"
 
 namespace Orthanc
 {
@@ -45,7 +47,10 @@ namespace Orthanc
     {
     }
 
-    virtual bool InvokeService(_OrthancPluginService service,
+    virtual bool InvokeService(SharedLibrary& plugin,
+                               _OrthancPluginService service,
                                const void* parameters) = 0;
   };
 }
+
+#endif
