@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -30,59 +30,22 @@
  **/
 
 
+#include "../../OrthancServer/PrecompiledHeadersServer.h"
 #include "OrthancPluginDatabase.h"
 
+#if ORTHANC_PLUGINS_ENABLED != 1
+#error The plugin support is disabled
+#endif
+
+
 #include "../../Core/OrthancException.h"
 #include "../../Core/Logging.h"
+#include "PluginsEnumerations.h"
 
 #include <cassert>
 
 namespace Orthanc
 {
-  static OrthancPluginResourceType Convert(ResourceType type)
-  {
-    switch (type)
-    {
-      case ResourceType_Patient:
-        return OrthancPluginResourceType_Patient;
-
-      case ResourceType_Study:
-        return OrthancPluginResourceType_Study;
-
-      case ResourceType_Series:
-        return OrthancPluginResourceType_Series;
-
-      case ResourceType_Instance:
-        return OrthancPluginResourceType_Instance;
-
-      default:
-        throw OrthancException(ErrorCode_InternalError);
-    }
-  }
-
-
-  static ResourceType Convert(OrthancPluginResourceType type)
-  {
-    switch (type)
-    {
-      case OrthancPluginResourceType_Patient:
-        return ResourceType_Patient;
-
-      case OrthancPluginResourceType_Study:
-        return ResourceType_Study;
-
-      case OrthancPluginResourceType_Series:
-        return ResourceType_Series;
-
-      case OrthancPluginResourceType_Instance:
-        return ResourceType_Instance;
-
-      default:
-        throw OrthancException(ErrorCode_InternalError);
-    }
-  }
-
-
   static FileInfo Convert(const OrthancPluginAttachment& attachment)
   {
     return FileInfo(attachment.uuid,
@@ -95,6 +58,16 @@ namespace Orthanc
   }
 
 
+  void OrthancPluginDatabase::CheckSuccess(OrthancPluginErrorCode code)
+  {
+    if (code != OrthancPluginErrorCode_Success)
+    {
+      errorDictionary_.LogError(code, true);
+      throw OrthancException(static_cast<ErrorCode>(code));
+    }
+  }
+
+
   void OrthancPluginDatabase::ResetAnswers()
   {
     type_ = _OrthancPluginDatabaseAnswerType_None;
@@ -111,7 +84,7 @@ namespace Orthanc
     if (type_ != _OrthancPluginDatabaseAnswerType_None &&
         type_ != _OrthancPluginDatabaseAnswerType_Int64)
     {
-      throw OrthancException(ErrorCode_Plugin);
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
 
     target.clear();
@@ -132,7 +105,7 @@ namespace Orthanc
     if (type_ != _OrthancPluginDatabaseAnswerType_None &&
         type_ != _OrthancPluginDatabaseAnswerType_String)
     {
-      throw OrthancException(ErrorCode_Plugin);
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
 
     target.clear();
@@ -162,7 +135,7 @@ namespace Orthanc
     }
     else
     {
-      throw OrthancException(ErrorCode_Plugin);
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
   }
 
@@ -181,15 +154,19 @@ namespace Orthanc
     }
     else
     {
-      throw OrthancException(ErrorCode_Plugin);
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
   }
 
 
-  OrthancPluginDatabase::OrthancPluginDatabase(const OrthancPluginDatabaseBackend& backend,
+  OrthancPluginDatabase::OrthancPluginDatabase(SharedLibrary& library,
+                                               PluginsErrorDictionary&  errorDictionary,
+                                               const OrthancPluginDatabaseBackend& backend,
                                                const OrthancPluginDatabaseExtensions* extensions,
                                                size_t extensionsSize,
                                                void *payload) : 
+    library_(library),
+    errorDictionary_(errorDictionary),
     type_(_OrthancPluginDatabaseAnswerType_None),
     backend_(backend),
     payload_(payload),
@@ -223,38 +200,26 @@ namespace Orthanc
     tmp.compressedSize = attachment.GetCompressedSize();
     tmp.compressedHash = attachment.GetCompressedMD5().c_str();
 
-    if (backend_.addAttachment(payload_, id, &tmp) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.addAttachment(payload_, id, &tmp));
   }
 
 
   void OrthancPluginDatabase::AttachChild(int64_t parent,
                                           int64_t child)
   {
-    if (backend_.attachChild(payload_, parent, child) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.attachChild(payload_, parent, child));
   }
 
 
   void OrthancPluginDatabase::ClearChanges()
   {
-    if (backend_.clearChanges(payload_) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.clearChanges(payload_));
   }
 
 
   void OrthancPluginDatabase::ClearExportedResources()
   {
-    if (backend_.clearExportedResources(payload_) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.clearExportedResources(payload_));
   }
 
 
@@ -262,12 +227,7 @@ namespace Orthanc
                                                 ResourceType type)
   {
     int64_t id;
-
-    if (backend_.createResource(&id, payload_, publicId.c_str(), Convert(type)) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.createResource(&id, payload_, publicId.c_str(), Plugins::Convert(type)));
     return id;
   }
 
@@ -275,29 +235,20 @@ namespace Orthanc
   void OrthancPluginDatabase::DeleteAttachment(int64_t id,
                                                FileContentType attachment)
   {
-    if (backend_.deleteAttachment(payload_, id, static_cast<int32_t>(attachment)) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.deleteAttachment(payload_, id, static_cast<int32_t>(attachment)));
   }
 
 
   void OrthancPluginDatabase::DeleteMetadata(int64_t id,
                                              MetadataType type)
   {
-    if (backend_.deleteMetadata(payload_, id, static_cast<int32_t>(type)) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.deleteMetadata(payload_, id, static_cast<int32_t>(type)));
   }
 
 
   void OrthancPluginDatabase::DeleteResource(int64_t id)
   {
-    if (backend_.deleteResource(payload_, id) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.deleteResource(payload_, id));
   }
 
 
@@ -315,7 +266,7 @@ namespace Orthanc
       std::string value;
       if (!LookupMetadata(value, id, *it))
       {
-        throw OrthancException(ErrorCode_Plugin);
+        throw OrthancException(ErrorCode_DatabasePlugin);
       }
 
       target[*it] = value;
@@ -323,16 +274,26 @@ namespace Orthanc
   }
 
 
+  void OrthancPluginDatabase::GetAllInternalIds(std::list<int64_t>& target,
+                                                ResourceType resourceType)
+  {
+    if (extensions_.getAllInternalIds == NULL)
+    {
+      LOG(ERROR) << "The database plugin does not implement the GetAllInternalIds primitive";
+      throw OrthancException(ErrorCode_DatabasePlugin);
+    }
+
+    ResetAnswers();
+    CheckSuccess(extensions_.getAllInternalIds(GetContext(), payload_, Plugins::Convert(resourceType)));
+    ForwardAnswers(target);
+  }
+
+
   void OrthancPluginDatabase::GetAllPublicIds(std::list<std::string>& target,
                                               ResourceType resourceType)
   {
     ResetAnswers();
-
-    if (backend_.getAllPublicIds(GetContext(), payload_, Convert(resourceType)) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.getAllPublicIds(GetContext(), payload_, Plugins::Convert(resourceType)));
     ForwardAnswers(target);
   }
 
@@ -346,12 +307,8 @@ namespace Orthanc
     {
       // This extension is available since Orthanc 0.9.4
       ResetAnswers();
-
-      if (extensions_.getAllPublicIdsWithLimit(GetContext(), payload_, Convert(resourceType), since, limit) != 0)
-      {
-        throw OrthancException(ErrorCode_Plugin);
-      }
-
+      CheckSuccess(extensions_.getAllPublicIdsWithLimit
+                   (GetContext(), payload_, Plugins::Convert(resourceType), since, limit));
       ForwardAnswers(target);
     }
     else
@@ -398,10 +355,7 @@ namespace Orthanc
     answerDone_ = &done;
     done = false;
 
-    if (backend_.getChanges(GetContext(), payload_, since, maxResults) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.getChanges(GetContext(), payload_, since, maxResults));
   }
 
 
@@ -409,12 +363,7 @@ namespace Orthanc
                                                     int64_t id)
   {
     ResetAnswers();
-
-    if (backend_.getChildrenInternalId(GetContext(), payload_, id) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.getChildrenInternalId(GetContext(), payload_, id));
     ForwardAnswers(target);
   }
 
@@ -423,12 +372,7 @@ namespace Orthanc
                                                   int64_t id)
   {
     ResetAnswers();
-
-    if (backend_.getChildrenPublicId(GetContext(), payload_, id) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.getChildrenPublicId(GetContext(), payload_, id));
     ForwardAnswers(target);
   }
 
@@ -443,10 +387,7 @@ namespace Orthanc
     answerDone_ = &done;
     done = false;
 
-    if (backend_.getExportedResources(GetContext(), payload_, since, maxResults) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.getExportedResources(GetContext(), payload_, since, maxResults));
   }
 
 
@@ -458,10 +399,7 @@ namespace Orthanc
     answerChanges_ = &target;
     answerDone_ = &ignored;
 
-    if (backend_.getLastChange(GetContext(), payload_) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.getLastChange(GetContext(), payload_));
   }
 
 
@@ -473,10 +411,7 @@ namespace Orthanc
     answerExportedResources_ = &target;
     answerDone_ = &ignored;
 
-    if (backend_.getLastExportedResource(GetContext(), payload_) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.getLastExportedResource(GetContext(), payload_));
   }
 
 
@@ -486,10 +421,7 @@ namespace Orthanc
     ResetAnswers();
     answerDicomMap_ = &map;
 
-    if (backend_.getMainDicomTags(GetContext(), payload_, id) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.getMainDicomTags(GetContext(), payload_, id));
   }
 
 
@@ -498,10 +430,11 @@ namespace Orthanc
     ResetAnswers();
     std::string s;
 
-    if (backend_.getPublicId(GetContext(), payload_, resourceId) != 0 ||
-        !ForwardSingleAnswer(s))
+    CheckSuccess(backend_.getPublicId(GetContext(), payload_, resourceId));
+
+    if (!ForwardSingleAnswer(s))
     {
-      throw OrthancException(ErrorCode_Plugin);
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
 
     return s;
@@ -511,12 +444,7 @@ namespace Orthanc
   uint64_t OrthancPluginDatabase::GetResourceCount(ResourceType resourceType)
   {
     uint64_t count;
-
-    if (backend_.getResourceCount(&count, payload_, Convert(resourceType)) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.getResourceCount(&count, payload_, Plugins::Convert(resourceType)));
     return count;
   }
 
@@ -524,25 +452,15 @@ namespace Orthanc
   ResourceType OrthancPluginDatabase::GetResourceType(int64_t resourceId)
   {
     OrthancPluginResourceType type;
-
-    if (backend_.getResourceType(&type, payload_, resourceId) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
-    return Convert(type);
+    CheckSuccess(backend_.getResourceType(&type, payload_, resourceId));
+    return Plugins::Convert(type);
   }
 
 
   uint64_t OrthancPluginDatabase::GetTotalCompressedSize()
   {
     uint64_t size;
-
-    if (backend_.getTotalCompressedSize(&size, payload_) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.getTotalCompressedSize(&size, payload_));
     return size;
   }
 
@@ -550,12 +468,7 @@ namespace Orthanc
   uint64_t OrthancPluginDatabase::GetTotalUncompressedSize()
   {
     uint64_t size;
-
-    if (backend_.getTotalUncompressedSize(&size, payload_) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.getTotalUncompressedSize(&size, payload_));
     return size;
   }
 
@@ -563,12 +476,7 @@ namespace Orthanc
   bool OrthancPluginDatabase::IsExistingResource(int64_t internalId)
   {
     int32_t existing;
-
-    if (backend_.isExistingResource(&existing, payload_, internalId) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.isExistingResource(&existing, payload_, internalId));
     return (existing != 0);
   }
 
@@ -576,12 +484,7 @@ namespace Orthanc
   bool OrthancPluginDatabase::IsProtectedPatient(int64_t internalId)
   {
     int32_t isProtected;
-
-    if (backend_.isProtectedPatient(&isProtected, payload_, internalId) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.isProtectedPatient(&isProtected, payload_, internalId));
     return (isProtected != 0);
   }
 
@@ -590,16 +493,12 @@ namespace Orthanc
                                                     int64_t id)
   {
     ResetAnswers();
-
-    if (backend_.listAvailableMetadata(GetContext(), payload_, id) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.listAvailableMetadata(GetContext(), payload_, id));
 
     if (type_ != _OrthancPluginDatabaseAnswerType_None &&
         type_ != _OrthancPluginDatabaseAnswerType_Int32)
     {
-      throw OrthancException(ErrorCode_Plugin);
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
 
     target.clear();
@@ -620,15 +519,12 @@ namespace Orthanc
   {
     ResetAnswers();
 
-    if (backend_.listAvailableAttachments(GetContext(), payload_, id) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.listAvailableAttachments(GetContext(), payload_, id));
 
     if (type_ != _OrthancPluginDatabaseAnswerType_None &&
         type_ != _OrthancPluginDatabaseAnswerType_Int32)
     {
-      throw OrthancException(ErrorCode_Plugin);
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
 
     target.clear();
@@ -650,14 +546,11 @@ namespace Orthanc
     OrthancPluginChange tmp;
     tmp.seq = change.GetSeq();
     tmp.changeType = static_cast<int32_t>(change.GetChangeType());
-    tmp.resourceType = Convert(change.GetResourceType());
+    tmp.resourceType = Plugins::Convert(change.GetResourceType());
     tmp.publicId = change.GetPublicId().c_str();
     tmp.date = change.GetDate().c_str();
 
-    if (backend_.logChange(payload_, &tmp) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.logChange(payload_, &tmp));
   }
 
 
@@ -665,7 +558,7 @@ namespace Orthanc
   {
     OrthancPluginExportedResource tmp;
     tmp.seq = resource.GetSeq();
-    tmp.resourceType = Convert(resource.GetResourceType());
+    tmp.resourceType = Plugins::Convert(resource.GetResourceType());
     tmp.publicId = resource.GetPublicId().c_str();
     tmp.modality = resource.GetModality().c_str();
     tmp.date = resource.GetDate().c_str();
@@ -674,10 +567,7 @@ namespace Orthanc
     tmp.seriesInstanceUid = resource.GetSeriesInstanceUid().c_str();
     tmp.sopInstanceUid = resource.GetSopInstanceUid().c_str();
 
-    if (backend_.logExportedResource(payload_, &tmp) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.logExportedResource(payload_, &tmp));
   }
 
     
@@ -687,10 +577,8 @@ namespace Orthanc
   {
     ResetAnswers();
 
-    if (backend_.lookupAttachment(GetContext(), payload_, id, static_cast<int32_t>(contentType)))
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.lookupAttachment
+                 (GetContext(), payload_, id, static_cast<int32_t>(contentType)));
 
     if (type_ == _OrthancPluginDatabaseAnswerType_None)
     {
@@ -704,7 +592,7 @@ namespace Orthanc
     }
     else
     {
-      throw OrthancException(ErrorCode_Plugin);
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
   }
 
@@ -714,47 +602,34 @@ namespace Orthanc
   {
     ResetAnswers();
 
-    if (backend_.lookupGlobalProperty(GetContext(), payload_, 
-                                      static_cast<int32_t>(property)))
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.lookupGlobalProperty
+                 (GetContext(), payload_, static_cast<int32_t>(property)));
 
     return ForwardSingleAnswer(target);
   }
 
 
-  void OrthancPluginDatabase::LookupIdentifier(std::list<int64_t>& target,
+  void OrthancPluginDatabase::LookupIdentifier(std::list<int64_t>& result,
+                                               ResourceType level,
                                                const DicomTag& tag,
+                                               IdentifierConstraintType type,
                                                const std::string& value)
   {
-    ResetAnswers();
+    if (extensions_.lookupIdentifier3 == NULL)
+    {
+      LOG(ERROR) << "The database plugin does not implement the LookupIdentifier3 primitive";
+      throw OrthancException(ErrorCode_DatabasePlugin);
+    }
 
     OrthancPluginDicomTag tmp;
     tmp.group = tag.GetGroup();
     tmp.element = tag.GetElement();
     tmp.value = value.c_str();
 
-    if (backend_.lookupIdentifier(GetContext(), payload_, &tmp) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
-    ForwardAnswers(target);
-  }
-
-
-  void OrthancPluginDatabase::LookupIdentifier(std::list<int64_t>& target,
-                                               const std::string& value)
-  {
     ResetAnswers();
-
-    if (backend_.lookupIdentifier2(GetContext(), payload_, value.c_str()) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
-    ForwardAnswers(target);
+    CheckSuccess(extensions_.lookupIdentifier3(GetContext(), payload_, Plugins::Convert(level),
+                                               &tmp, Plugins::Convert(type)));
+    ForwardAnswers(result);
   }
 
 
@@ -763,12 +638,7 @@ namespace Orthanc
                                              MetadataType type)
   {
     ResetAnswers();
-
-    if (backend_.lookupMetadata(GetContext(), payload_, id, static_cast<int32_t>(type)))
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.lookupMetadata(GetContext(), payload_, id, static_cast<int32_t>(type)));
     return ForwardSingleAnswer(target);
   }
 
@@ -777,12 +647,7 @@ namespace Orthanc
                                            int64_t resourceId)
   {
     ResetAnswers();
-
-    if (backend_.lookupParent(GetContext(), payload_, resourceId))
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.lookupParent(GetContext(), payload_, resourceId));
     return ForwardSingleAnswer(parentId);
   }
 
@@ -793,10 +658,7 @@ namespace Orthanc
   {
     ResetAnswers();
 
-    if (backend_.lookupResource(GetContext(), payload_, publicId.c_str()))
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.lookupResource(GetContext(), payload_, publicId.c_str()));
 
     if (type_ == _OrthancPluginDatabaseAnswerType_None)
     {
@@ -811,7 +673,7 @@ namespace Orthanc
     }
     else
     {
-      throw OrthancException(ErrorCode_Plugin);
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
   }
 
@@ -819,12 +681,7 @@ namespace Orthanc
   bool OrthancPluginDatabase::SelectPatientToRecycle(int64_t& internalId)
   {
     ResetAnswers();
-
-    if (backend_.selectPatientToRecycle(GetContext(), payload_))
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.selectPatientToRecycle(GetContext(), payload_));
     return ForwardSingleAnswer(internalId);
   }
 
@@ -833,12 +690,7 @@ namespace Orthanc
                                                      int64_t patientIdToAvoid)
   {
     ResetAnswers();
-
-    if (backend_.selectPatientToRecycle2(GetContext(), payload_, patientIdToAvoid))
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
-
+    CheckSuccess(backend_.selectPatientToRecycle2(GetContext(), payload_, patientIdToAvoid));
     return ForwardSingleAnswer(internalId);
   }
 
@@ -846,11 +698,20 @@ namespace Orthanc
   void OrthancPluginDatabase::SetGlobalProperty(GlobalProperty property,
                                                 const std::string& value)
   {
-    if (backend_.setGlobalProperty(payload_, static_cast<int32_t>(property), 
-                                   value.c_str()) != 0)
+    CheckSuccess(backend_.setGlobalProperty
+                 (payload_, static_cast<int32_t>(property), value.c_str()));
+  }
+
+
+  void OrthancPluginDatabase::ClearMainDicomTags(int64_t id)
+  {
+    if (extensions_.clearMainDicomTags == NULL)
     {
-      throw OrthancException(ErrorCode_Plugin);
+      LOG(ERROR) << "Your custom index plugin does not implement the ClearMainDicomTags() extension";
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
+
+    CheckSuccess(extensions_.clearMainDicomTags(payload_, id));
   }
 
 
@@ -858,25 +719,25 @@ namespace Orthanc
                                               const DicomTag& tag,
                                               const std::string& value)
   {
-    int32_t status;
     OrthancPluginDicomTag tmp;
     tmp.group = tag.GetGroup();
     tmp.element = tag.GetElement();
     tmp.value = value.c_str();
 
-    if (tag.IsIdentifier())
-    {
-      status = backend_.setIdentifierTag(payload_, id, &tmp);
-    }
-    else
-    {
-      status = backend_.setMainDicomTag(payload_, id, &tmp);
-    }
+    CheckSuccess(backend_.setMainDicomTag(payload_, id, &tmp));
+  }
 
-    if (status != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+
+  void OrthancPluginDatabase::SetIdentifierTag(int64_t id,
+                                               const DicomTag& tag,
+                                               const std::string& value)
+  {
+    OrthancPluginDicomTag tmp;
+    tmp.group = tag.GetGroup();
+    tmp.element = tag.GetElement();
+    tmp.value = value.c_str();
+
+    CheckSuccess(backend_.setIdentifierTag(payload_, id, &tmp));
   }
 
 
@@ -884,21 +745,15 @@ namespace Orthanc
                                           MetadataType type,
                                           const std::string& value)
   {
-    if (backend_.setMetadata(payload_, id, static_cast<int32_t>(type), 
-                             value.c_str()) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.setMetadata
+                 (payload_, id, static_cast<int32_t>(type), value.c_str()));
   }
 
 
   void OrthancPluginDatabase::SetProtectedPatient(int64_t internalId, 
                                                   bool isProtected)
   {
-    if (backend_.setProtectedPatient(payload_, internalId, isProtected) != 0)
-    {
-      throw OrthancException(ErrorCode_Plugin);
-    }
+    CheckSuccess(backend_.setProtectedPatient(payload_, internalId, isProtected));
   }
 
 
@@ -907,44 +762,47 @@ namespace Orthanc
   private:
     const OrthancPluginDatabaseBackend& backend_;
     void* payload_;
+    PluginsErrorDictionary&  errorDictionary_;
+
+    void CheckSuccess(OrthancPluginErrorCode code)
+    {
+      if (code != OrthancPluginErrorCode_Success)
+      {
+        errorDictionary_.LogError(code, true);
+        throw OrthancException(static_cast<ErrorCode>(code));
+      }
+    }
 
   public:
     Transaction(const OrthancPluginDatabaseBackend& backend,
-                void* payload) :
+                void* payload,
+                PluginsErrorDictionary&  errorDictionary) :
       backend_(backend),
-      payload_(payload)
+      payload_(payload),
+      errorDictionary_(errorDictionary)
     {
     }
 
     virtual void Begin()
     {
-      if (backend_.startTransaction(payload_) != 0)
-      {
-        throw OrthancException(ErrorCode_Plugin);
-      }
+      CheckSuccess(backend_.startTransaction(payload_));
     }
 
     virtual void Rollback()
     {
-      if (backend_.rollbackTransaction(payload_) != 0)
-      {
-        throw OrthancException(ErrorCode_Plugin);
-      }
+      CheckSuccess(backend_.rollbackTransaction(payload_));
     }
 
     virtual void Commit()
     {
-      if (backend_.commitTransaction(payload_) != 0)
-      {
-        throw OrthancException(ErrorCode_Plugin);
-      }
+      CheckSuccess(backend_.commitTransaction(payload_));
     }
   };
 
 
   SQLite::ITransaction* OrthancPluginDatabase::StartTransaction()
   {
-    return new Transaction(backend_, payload_);
+    return new Transaction(backend_, payload_, errorDictionary_);
   }
 
 
@@ -963,21 +821,21 @@ namespace Orthanc
         
       case _OrthancPluginDatabaseAnswerType_RemainingAncestor:
       {
-        ResourceType type = Convert(static_cast<OrthancPluginResourceType>(answer.valueInt32));
+        ResourceType type = Plugins::Convert(static_cast<OrthancPluginResourceType>(answer.valueInt32));
         listener.SignalRemainingAncestor(type, answer.valueString);
         break;
       }
       
       case _OrthancPluginDatabaseAnswerType_DeletedResource:
       {
-        ResourceType type = Convert(static_cast<OrthancPluginResourceType>(answer.valueInt32));
+        ResourceType type = Plugins::Convert(static_cast<OrthancPluginResourceType>(answer.valueInt32));
         ServerIndexChange change(ChangeType_Deleted, type, answer.valueString);
         listener.SignalChange(change);
         break;
       }
 
       default:
-        throw OrthancException(ErrorCode_Plugin);
+        throw OrthancException(ErrorCode_DatabasePlugin);
     }
   }
 
@@ -987,11 +845,7 @@ namespace Orthanc
     if (extensions_.getDatabaseVersion != NULL)
     {
       uint32_t version;
-      if (extensions_.getDatabaseVersion(&version, payload_) != 0)
-      {
-        throw OrthancException(ErrorCode_Plugin);
-      }
-
+      CheckSuccess(extensions_.getDatabaseVersion(&version, payload_));
       return version;
     }
     else
@@ -1009,11 +863,22 @@ namespace Orthanc
   {
     if (extensions_.upgradeDatabase != NULL)
     {
-      if (extensions_.upgradeDatabase(
-            payload_, targetVersion, 
-            reinterpret_cast<OrthancPluginStorageArea*>(&storageArea)) != 0)
+      Transaction transaction(backend_, payload_, errorDictionary_);
+      transaction.Begin();
+
+      OrthancPluginErrorCode code = extensions_.upgradeDatabase(
+        payload_, targetVersion, 
+        reinterpret_cast<OrthancPluginStorageArea*>(&storageArea));
+
+      if (code == OrthancPluginErrorCode_Success)
       {
-        throw OrthancException(ErrorCode_Plugin);
+        transaction.Commit();
+      }
+      else
+      {
+        transaction.Rollback();
+        errorDictionary_.LogError(code, true);
+        throw OrthancException(static_cast<ErrorCode>(code));
       }
     }
   }
@@ -1023,7 +888,7 @@ namespace Orthanc
   {
     if (answer.type == _OrthancPluginDatabaseAnswerType_None)
     {
-      throw OrthancException(ErrorCode_Plugin);
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
 
     if (answer.type == _OrthancPluginDatabaseAnswerType_DeletedAttachment ||
@@ -1078,13 +943,13 @@ namespace Orthanc
 
         default:
           LOG(ERROR) << "Unhandled type of answer for custom index plugin: " << answer.type;
-          throw OrthancException(ErrorCode_Plugin);
+          throw OrthancException(ErrorCode_DatabasePlugin);
       }
     }
     else if (type_ != answer.type)
     {
       LOG(ERROR) << "Error in the plugin protocol: Cannot change the answer type";
-      throw OrthancException(ErrorCode_Plugin);
+      throw OrthancException(ErrorCode_DatabasePlugin);
     }
 
     switch (answer.type)
@@ -1104,7 +969,7 @@ namespace Orthanc
       case _OrthancPluginDatabaseAnswerType_Resource:
       {
         OrthancPluginResourceType type = static_cast<OrthancPluginResourceType>(answer.valueInt32);
-        answerResources_.push_back(std::make_pair(answer.valueInt64, Convert(type)));
+        answerResources_.push_back(std::make_pair(answer.valueInt64, Plugins::Convert(type)));
         break;
       }
 
@@ -1121,7 +986,7 @@ namespace Orthanc
       {
         const OrthancPluginDicomTag& tag = *reinterpret_cast<const OrthancPluginDicomTag*>(answer.valueGeneric);
         assert(answerDicomMap_ != NULL);
-        answerDicomMap_->SetValue(tag.group, tag.element, std::string(tag.value));
+        answerDicomMap_->SetValue(tag.group, tag.element, std::string(tag.value), false);
         break;
       }
 
@@ -1129,7 +994,7 @@ namespace Orthanc
       {
         if (answer.valueString == NULL)
         {
-          throw OrthancException(ErrorCode_Plugin);
+          throw OrthancException(ErrorCode_DatabasePlugin);
         }
 
         if (type_ == _OrthancPluginDatabaseAnswerType_None)
@@ -1139,7 +1004,7 @@ namespace Orthanc
         }
         else if (type_ != _OrthancPluginDatabaseAnswerType_String)
         {
-          throw OrthancException(ErrorCode_Plugin);
+          throw OrthancException(ErrorCode_DatabasePlugin);
         }
 
         answerStrings_.push_back(std::string(answer.valueString));
@@ -1155,7 +1020,7 @@ namespace Orthanc
         }
         else if (*answerDone_)
         {
-          throw OrthancException(ErrorCode_Plugin);
+          throw OrthancException(ErrorCode_DatabasePlugin);
         }
         else
         {
@@ -1164,7 +1029,7 @@ namespace Orthanc
           answerChanges_->push_back
             (ServerIndexChange(change.seq,
                                static_cast<ChangeType>(change.changeType),
-                               Convert(change.resourceType),
+                               Plugins::Convert(change.resourceType),
                                change.publicId,
                                change.date));                                   
         }
@@ -1181,7 +1046,7 @@ namespace Orthanc
         }
         else if (*answerDone_)
         {
-          throw OrthancException(ErrorCode_Plugin);
+          throw OrthancException(ErrorCode_DatabasePlugin);
         }
         else
         {
@@ -1190,7 +1055,7 @@ namespace Orthanc
           assert(answerExportedResources_ != NULL);
           answerExportedResources_->push_back
             (ExportedResource(exported.seq,
-                              Convert(exported.resourceType),
+                              Plugins::Convert(exported.resourceType),
                               exported.publicId,
                               exported.modality,
                               exported.date,
@@ -1205,7 +1070,7 @@ namespace Orthanc
 
       default:
         LOG(ERROR) << "Unhandled type of answer for custom index plugin: " << answer.type;
-        throw OrthancException(ErrorCode_Plugin);
+        throw OrthancException(ErrorCode_DatabasePlugin);
     }
   }
 }
