@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -30,8 +30,14 @@
  **/
 
 
+#include "../../OrthancServer/PrecompiledHeadersServer.h"
 #include "PluginsManager.h"
 
+#if ORTHANC_PLUGINS_ENABLED != 1
+#error The plugin support is disabled
+#endif
+
+
 #include "../../Core/Toolbox.h"
 #include "../../Core/HttpServer/HttpOutput.h"
 #include "../../Core/Logging.h"
@@ -42,7 +48,7 @@
 
 #ifdef WIN32
 #define PLUGIN_EXTENSION ".dll"
-#elif defined(__linux) || defined(__FreeBSD_kernel__) || defined(__FreeBSD__)
+#elif defined(__linux__) || defined(__FreeBSD_kernel__) || defined(__FreeBSD__)
 #define PLUGIN_EXTENSION ".so"
 #elif defined(__APPLE__) && defined(__MACH__)
 #define PLUGIN_EXTENSION ".dylib"
@@ -53,6 +59,19 @@
 
 namespace Orthanc
 {
+  PluginsManager::Plugin::Plugin(PluginsManager& pluginManager,
+                                 const std::string& path) : 
+    library_(path),
+    pluginManager_(pluginManager)
+  {
+    memset(&context_, 0, sizeof(context_));
+    context_.pluginsManager = this;
+    context_.orthancVersion = ORTHANC_VERSION;
+    context_.Free = ::free;
+    context_.InvokeService = InvokeService;
+  }
+
+
   static void CallInitialize(SharedLibrary& plugin,
                              const OrthancPluginContext& context)
   {
@@ -156,15 +175,15 @@ namespace Orthanc
         break;
     }
 
-    PluginsManager* that = reinterpret_cast<PluginsManager*>(context->pluginsManager);
+    Plugin* that = reinterpret_cast<Plugin*>(context->pluginsManager);
 
     for (std::list<IPluginServiceProvider*>::iterator
-           it = that->serviceProviders_.begin(); 
-         it != that->serviceProviders_.end(); ++it)
+           it = that->GetPluginManager().serviceProviders_.begin(); 
+         it != that->GetPluginManager().serviceProviders_.end(); ++it)
     {
       try
       {
-        if ((*it)->InvokeService(service, params))
+        if ((*it)->InvokeService(that->GetSharedLibrary(), service, params))
         {
           return OrthancPluginErrorCode_Success;
         }
@@ -172,7 +191,11 @@ namespace Orthanc
       catch (OrthancException& e)
       {
         // This service provider has failed
-        LOG(ERROR) << "Exception while invoking a plugin service: " << e.What();
+        if (e.GetErrorCode() != ErrorCode_UnknownResource)  // This error code is valid in plugins
+        {
+          LOG(ERROR) << "Exception while invoking plugin service " << service << ": " << e.What();
+        }
+
         return static_cast<OrthancPluginErrorCode>(e.GetErrorCode());
       }
     }
@@ -184,11 +207,6 @@ namespace Orthanc
 
   PluginsManager::PluginsManager()
   {
-    memset(&context_, 0, sizeof(context_));
-    context_.pluginsManager = this;
-    context_.orthancVersion = ORTHANC_VERSION;
-    context_.Free = ::free;
-    context_.InvokeService = InvokeService;
   }
 
   PluginsManager::~PluginsManager()
@@ -200,7 +218,7 @@ namespace Orthanc
         LOG(WARNING) << "Unregistering plugin '" << it->first
                      << "' (version " << it->second->GetVersion() << ")";
 
-        CallFinalize(it->second->GetLibrary());
+        CallFinalize(it->second->GetSharedLibrary());
         delete it->second;
       }
     }
@@ -230,27 +248,27 @@ namespace Orthanc
       return;
     }
 
-    std::auto_ptr<Plugin> plugin(new Plugin(path));
+    std::auto_ptr<Plugin> plugin(new Plugin(*this, path));
 
-    if (!IsOrthancPlugin(plugin->GetLibrary()))
+    if (!IsOrthancPlugin(plugin->GetSharedLibrary()))
     {
-      LOG(ERROR) << "Plugin " << plugin->GetLibrary().GetPath()
+      LOG(ERROR) << "Plugin " << plugin->GetSharedLibrary().GetPath()
                  << " does not declare the proper entry functions";
       throw OrthancException(ErrorCode_SharedLibrary);
     }
 
-    std::string name(CallGetName(plugin->GetLibrary()));
+    std::string name(CallGetName(plugin->GetSharedLibrary()));
     if (plugins_.find(name) != plugins_.end())
     {
       LOG(ERROR) << "Plugin '" << name << "' already registered";
       throw OrthancException(ErrorCode_SharedLibrary);
     }
 
-    plugin->SetVersion(CallGetVersion(plugin->GetLibrary()));
+    plugin->SetVersion(CallGetVersion(plugin->GetSharedLibrary()));
     LOG(WARNING) << "Registering plugin '" << name
                  << "' (version " << plugin->GetVersion() << ")";
 
-    CallInitialize(plugin->GetLibrary(), context_);
+    CallInitialize(plugin->GetSharedLibrary(), plugin->GetContext());
 
     plugins_[name] = plugin.release();
   }
@@ -333,4 +351,9 @@ namespace Orthanc
     }
   }
 
+  
+  std::string PluginsManager::GetPluginName(SharedLibrary& library)
+  {
+    return CallGetName(library);
+  }
 }
