@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012-2015 Sebastien Jodogne, Medical Physics
+ * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -32,6 +32,8 @@
 
 #pragma once
 
+#if ORTHANC_PLUGINS_ENABLED == 1
+
 #include "../../Core/OrthancException.h"
 
 #include <boost/noncopyable.hpp>
@@ -42,7 +44,7 @@
 
 namespace Orthanc
 {
-  class SharedLibrary : boost::noncopyable
+  class SharedLibrary : public boost::noncopyable
   {
   public:
 #if defined(_WIN32)
@@ -72,3 +74,5 @@ namespace Orthanc
     FunctionPointer GetFunction(const std::string& name);
   };
 }
+
+#endif
